@@ -29,13 +29,13 @@ our @EXPORT = qw(
 our @ISA = qw(Exporter);
 
 # Set the package version. 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 # Load the Perl modules.
 use POSIX qw(ttyname);
 
 # Use inline C code.
-#use Inline 'C';
+# use Inline 'C';
 
 # Define the Perl BEGIN block.
 BEGIN {
@@ -65,8 +65,7 @@ my %termhash = (
     "foot"                   => "foot",                    # know, but not tested
     "gnome-terminal-server"  => "Gnome Terminal",          # successful tested
     "guake"                  => "Guake Terminal",          # successful tested
-    "hterm"                  => "HTerm",                   # know, but not tested
-    "hyper"                  => "Hyper",                   # know, but not tested
+    "hyper"                  => "Hyper",                   # successful tested
     "kitty"                  => "kitty",                   # successful tested
     "konsole"                => "Konsole",                 # successful tested
     "lilyterm"               => "LilyTerm",                # successful tested
@@ -82,22 +81,25 @@ my %termhash = (
     "rxvt-unicode"           => "rxvt-unicode",            # successful tested
     "rxvt-unicode-truecolor" => "rxvt-unicode-truecolor",  # know, but not tested
     "sakura"                 => "Sakura",                  # successful tested
+    "screen"                 => "SCREEN",                  # successful tested
     "stterm"                 => "stterm",                  # successful tested
-    "tabby"                  => "Tabby",                   # know, but not tested
+    "tabby"                  => "Tabby",                   # successful tested
     "terminal"               => "Terminal",                # know, but not tested
     "terminator"             => "Terminator",              # successful tested
     "terminology"            => "Terminology",             # successful tested
+    "terminix"               => "Terminix",                # know, but not tested
     "termit"                 => "Termit",                  # successful tested
     "termite"                => "Termite",                 # know, but not tested
     "tilda"                  => "Tilda",                   # successful tested
-    "tilix"                  => "Tilix",                   # know, but not tested
+    "tilix"                  => "Tilix",                   # successful tested
+    "tmux"                   => "tmux",                    # successful tested
     "urxvt"                  => "urxvt",                   # successful tested
     "uxterm"                 => "UXTerm",                  # successful tested
-    "wezterm"                => "WezTerm",                 # know, but not tested
+    "wezterm"                => "WezTerm",                 # Test fails somehow
     "xfce4-terminal"         => "Xfce4-Terminal-Emulator", # successful tested
     "xiterm+thai"            => "xiterm+thai",             # successful tested
     "xterm"                  => "XTerm",                   # successful tested
-    "xvt"                    => "xvt",                     # know, but not tested
+    "xvt"                    => "Xvt",                     # know, but not tested
     "yakuake"                => "Yakuake"                  # successful tested
 );
 
@@ -109,8 +111,8 @@ my %termhash = (
 # by a logical or operation and using \s from regular expressions. Anchors are #
 # begin of string and end of of string.                                        #
 #                                                                              #
-# @argument: $_[0]  string to trim  (scalar)                                   #
-# @return:   $str   trimmed string  (scalar)                                   #
+# @argument: $_[0]  String to trim  (scalar)                                   #
+# @return:   $str   Trimmed string  (scalar)                                   #
 #------------------------------------------------------------------------------# 
 sub trim {
     # Assign the function argument to the local string variable $str.  
@@ -125,12 +127,16 @@ sub trim {
 # Subroutine search_brackets                                                   #
 #                                                                              #
 # Description:                                                                 #
-# Adds [ and ] to searchstring for grep.                                       # 
+# The subroutine adds the square brackets [ and ] to the searchstring for      #
+# use with grep. E.g. shell becomes [s]hell.                                   # 
+#                                                                              #
+# @argument: $_[0]  Searchstring                       (scalar)                #
+# @return:   $str   Searchstring with square brackets  (scalar)                #
 #------------------------------------------------------------------------------# 
 sub search_brackets {
-    # Assign the function argument to the local variable.  
+    # Assign the subroutine argument to the local variable.  
     my $str = $_[0];
-    # Add square brackets to the given string.
+    # Add square brackets [ and ] to the given string.
     substr($str, 0, 0) = '[';
     substr($str, 2, 0) = ']';
     # Return the modified string.
@@ -144,8 +150,8 @@ sub search_brackets {
 # Read a file in one chunk. The retrieved content is stored in a string        #
 # variable.                                                                    #
 #                                                                              #
-# @argument: $_[0]     filename      (scalar)                                  #
-# @return:   $content  file content  (scalar)                                  #
+# @argument: $_[0]     Filename      (scalar)                                  #
+# @return:   $content  File content  (scalar)                                  #
 #------------------------------------------------------------------------------# 
 sub read_file {
     # Assign the function argument to the local variable.  
@@ -164,15 +170,18 @@ sub read_file {
 # Subroutine login_users                                                       #
 #                                                                              #
 # Description:                                                                 #
-# Write logged in user to array.                                               #
+# Split the logged-in users by sperator space and store them to an array.      #
+#                                                                              #
+# @argument: None                                                              #
+# @return:   $user_arr  Array with the logged-in users  (array)                #
 #------------------------------------------------------------------------------# 
 sub login_users {
-    # Assign the function argument to the local variable.  
+    # Get the logged-in users from Linux command users.   
     my $user = `users`;
-    # Write logged-in user to array.
-    my @user_arr = split ' ', $user;
-    # Return the array.
-    return @user_arr;
+    # Split the logged-in users and store them to the array.
+    my @user_array = split ' ', $user;
+    # Return the array with the logged-in users.
+    return @user_array;
 }
 
 #------------------------------------------------------------------------------# 
@@ -186,7 +195,7 @@ sub login_users {
 # last step douple entries are removed from the array.                         #
 #                                                                              #
 # @argument: None                                                              #
-# @return:   @login_shells  array with valid login shells  (array)             #
+# @return:   @login_shells  Array with valid login shells  (array)             #
 #------------------------------------------------------------------------------# 
 sub login_shells {
     # Declare the login shells array.
@@ -210,7 +219,7 @@ sub login_shells {
     # Remove the double entries from the array.
     my %login_hash = map({$_, 1} @login_shells);
     @login_shells = keys %login_hash;
-    # Return the array with the login shells.
+    # Return the array with the unique login shells.
     return @login_shells;
 };
 
@@ -219,8 +228,9 @@ sub login_shells {
 #                                                                              #
 # Description:                                                                 #
 # Determine the PPID of the calling Perl script using the Linux command ps.    # 
-# Argument: PID                                                                # 
-# Returns:  PPID                                                               # 
+#                                                                              #
+# @argument: PID                                                               # 
+# @return:   PPID                                                              # 
 #------------------------------------------------------------------------------# 
 sub get_ppid {
     # Assign the subroutine argument to the local variable $pid.
@@ -244,8 +254,8 @@ sub get_ppid {
 # Use the Linux command ps to get the command line of the process which is     #
 # related to the terminal in use.                                              #
 #                                                                              #
-# @argument  $_[0]      PPID          (scalar)                                 #
-# @return    $termproc  command line  (scalar)                                 #
+# @argument  $_[0]      PPID                   (scalar)                        #
+# @return    $termproc  Terminal command line  (scalar)                        #
 #------------------------------------------------------------------------------# 
 sub terminal_command_line {
     # Assign the subroutine argument to the local variable $ppid.
@@ -262,19 +272,28 @@ sub terminal_command_line {
 # Subroutine user_shell                                                        #
 #                                                                              #
 # Description:                                                                 #
-# Get user login shell.                                                        #
+# Get the user login shells.                                                   #
+#                                                                              #
+# @arguments: $_[0]     Content of passwd       (scalar)                       # 
+#             @{$_[1]}  User array              (array)                        #
+#             @{$_[2]}  Shell array             (array)                        #
+# @return:    @match    Array with the matches  (array)                        #
 #------------------------------------------------------------------------------# 
 sub user_shell {
-    # Assign the subroutine argument to the local variable $content.
-    my $content = $_[0];
-    my @user_arr = @{$_[1]};
-    my @shell_arr = @{$_[2]};
-    my @match;
-    # Loop over the array with the lines of $content.
-    foreach (split '\n', $content) {
+    # Assign the subroutine arguments to the local variables.
+    my $passwd_content = $_[0];
+    my @user_array = @{$_[1]};
+    my @shell_array = @{$_[2]};
+    # Declare the array @match.
+    my @match = ();
+    # Loop over the array with the lines of $passwd_content.
+    foreach (split '\n', $passwd_content) {
+        # Get user and shell from each line of content.
         my ($user) = $_ =~ /^(.*?):.*/;
         my ($shell) = $_ =~ /^.*\/(.*)$/;
-        if (grep(/^$user$/, @user_arr) && grep(/^$shell$/, @shell_arr)) {
+        # Check user and shall against the given arrays.
+        if (grep(/^$user$/, @user_array) && grep(/^$shell$/, @shell_array)) {
+            # Add line to array @match.
             push(@match, $_);
         };
     };
@@ -287,28 +306,41 @@ sub user_shell {
 #                                                                              #
 # Description:                                                                 #
 # Get the pseudo terminal slave, which is related to the executed Perl script. #
+#                                                                              #
+# @argument: @{$_[0]}  Array with the matches    (array)                       # 
+# @return:   @result   Array with the processes  (array)                       #
 #------------------------------------------------------------------------------# 
 sub user_term {
-    # Assign the subroutine argument to the local variable $content.
+    # Assign the subroutine argument to the local variable.
     my @match = @{$_[0]};
     # Declare the array @result.
     my @result;
     # Get the terminal path.
-    #my $term_path = TermPath();
+    # my $term_path = TermPath();
     my $fileno = fileno(STDIN);
     my $term_path = ttyname($fileno);
+    # Check if $term_path is defined, otherwise set $term_path to "".
+    $term_path = (defined $term_path ? $term_path : "");
+    # Extract the pseudo terminal slave. 
     my ($term) = $term_path =~ /^.*\/dev\/(pts\/\d+)$/;
+    # Return if $term is empty.
     if (!defined $term) {
+        # Return empty array. 
         return ();
     };
+    # Add square brackets to searchstring.
     $term = search_brackets($term);
-    # Loop over the matches.
+    # Loop over the matches (lines of passwd).
     foreach (@match) {
+        # Get user and shell from given line. 
         my ($user) = $_ =~ /^(.*?):.*/;
         my ($shell) = $_ =~ /^.*\/(.*)$/;
+        # Add square brackets to searchstrings.
         $user = search_brackets($user);
         $shell = search_brackets($shell);
+        # Search for process which is matching user, shell and terminal.
         my $process = `ps aux | grep $shell | grep $user | grep $term`;
+        # Check if result is not empty.
         if ($process ne '') {
             # Add process to array @result.
             push(@result, $process);
@@ -322,17 +354,22 @@ sub user_term {
 # Subroutine process_command_line                                              #
 #                                                                              #
 # Description:                                                                 #
-# Identify the process command of the terminal in use.                         #
+# Identify the process command line of the terminal in use.                    #
+#                                                                              #
+# @argument: None                                                              # 
+# @return:   $terminal  Process command line  (scalar)                         #
 #------------------------------------------------------------------------------# 
 sub process_command_line {
-    # Declare the return variable.
+    # Declare the return variable $terminal.
     my $terminal;
+    # Set the filename.
+    my $filename = "/etc/passwd";
     # Get the logged in users.
     my @user_arr = login_users();
     # Get the available login shells.
     my @shell_arr = login_shells();
     # Read the file /etc/passwd in variable $content.
-    my $content = read_file("/etc/passwd");
+    my $content = read_file($filename);
     # Create the array with user and shell.
     my @match = user_shell($content, \@user_arr, \@shell_arr);
     # Create the array with user and term.
@@ -355,7 +392,13 @@ sub process_command_line {
 };
 
 #------------------------------------------------------------------------------# 
-# Subroutine remove_arguments                                                  # 
+# Subroutine remove_arguments                                                  #
+#                                                                              #
+# Description:                                                                 #
+# Remove arguments which starts with -/-- from the array.                      #
+#                                                                              #
+# @argument: @{$_[0]}   Array with the parts of the command line  (array)      # 
+# @return:   $terminal  Process command line                      (scalar)     #
 #------------------------------------------------------------------------------# 
 sub remove_arguments {
     # Assign subroutine argument to the local array.
@@ -364,8 +407,11 @@ sub remove_arguments {
     my @new_parts;
     # Remove all arguments from the old array and store them in the new array.
     foreach (@old_parts) {
+        # Check if element is an argument. 
         if (substr($_, 0, 1) ne "-") {
+            # Check if element is a valid program
             if (system("which $_ >/dev/null 2>&1") == 0) { 
+                # Add valid program to array.  
                 push(@new_parts, $_)
             }; 
         };
@@ -375,10 +421,13 @@ sub remove_arguments {
 }
 
 #------------------------------------------------------------------------------# 
-# Subroutine which_terminal                                                    # 
+# Subroutine whichterminalami                                                  # 
 #                                                                              #
 # Description:                                                                 #
 # Identify the terminal name.                                                  #
+#                                                                              #
+# @argument: $_[0]      Output flag    (array)                                 # 
+# @return:   $terminal  Terminal name  (scalar)                                #
 #------------------------------------------------------------------------------# 
 sub whichterminalami {
     # Get the output flag from the subroutine argument.
@@ -411,7 +460,7 @@ sub whichterminalami {
             $terminal_path = $parts[1];
         };
     };
-    # Remove the directory from the path.
+    # Remove the path from the Terminal name.
     my $terminal_name = $terminal_path =~ s{^.*/}{}r;
     # Lookup terminal name in the terminal hash.
     if (defined $termhash{$terminal_name}) {
@@ -422,12 +471,14 @@ sub whichterminalami {
         $flag = $OutputFormat;
     };
     # Return terminal string based on flag setting.
-    if ($flag eq "PROC") {
-        $terminal = (defined $terminal_process ? $terminal_process : $str);
+    if ($flag eq "") {
+        $terminal = (defined $terminal_name ? $terminal_name : $str);
+    } elsif ($flag eq "FTN") {
+        $terminal = (defined $terminal_ftn ? $terminal_ftn : $terminal_name);
     } elsif ($flag eq "PATH") {
         $terminal = (defined $terminal_path ? $terminal_path : $str);
-    } elsif ($flag eq "FTN") {
-        $terminal = (defined $terminal_ftn ? $terminal_ftn : $str);
+    } elsif ($flag eq "PROC") {
+        $terminal = (defined $terminal_process ? $terminal_process : $str);
     } else {
         $terminal = (defined $terminal_name ? $terminal_name : $str);
     };
@@ -483,7 +534,7 @@ or like this
 
   # Identify the terminal emulator in use.
   whichterminalami();                      # Standard method call
-  Terminal::Identify::whichterminalami() ; # Alternate method call
+  Terminal::Identify::whichterminalami(); # Alternate method call
 
 =head1 USAGE
 
@@ -525,15 +576,15 @@ arguments C<"PROC">, C<"PATH"> and C<"FTN">.
 
 =item * "PROC"
 
-  PROC => Full process of the terminal emulator in use. 
+  PROC => Full process of the terminal emulator which is used. 
 
 =item * "PATH"
 
-  PATH => Path to the location of the terminal emulator in use.
+  PATH => Path to the location of the terminal emulator which is used.
 
 =item * "FTN"
 
-  FTN => Friendly terminal name of the terminal emulator in use.
+  FTN => Friendly terminal name of the terminal emulator which is used.
 
 =back
 
@@ -599,14 +650,6 @@ arguments C<"PROC">, C<"PATH"> and C<"FTN">.
   $terminal = whichterminalami();
   print $terminal . "\n";
 
-=head1 ERROR CODES
-
-No error codes yet
-
-=head1 NOTES
-
-No notes yet
-
 =head1 SYSTEM COMPATIBILITY
 
 The module should work on B<Linux> and on B<Unix> or B<Unix-like>
@@ -622,14 +665,14 @@ Following Linux commands should be available for functionality:
 
 =item * users
 
-=item * shells
-
-=item * passwd
-
 =item * which
 
 =item * grep
 
+=item * shells
+
+=item * passwd
+
 =back
 
 =head1 METHOD ALIASES
@@ -656,8 +699,12 @@ Terminal emulators tested so far with the package:
 
 =item * Eterm
 
+=item * Gnome Terminal
+
 =item * Guake Terminal
 
+=item * Hyper
+
 =item * kitty
 
 =item * Konsole
@@ -678,6 +725,10 @@ Terminal emulators tested so far with the package:
 
 =item * Sakura
 
+=item * SCREEN
+
+=item * Tabby
+
 =item * Terminator
 
 =item * Terminology
@@ -688,6 +739,8 @@ Terminal emulators tested so far with the package:
 
 =item * Tilix
 
+=item * tmux
+
 =item * UXTerm
 
 =item * Xfce4-Terminal-Emulator
@@ -709,13 +762,26 @@ system files C</etc/shells> and C</etc/passwd> must be exist.
 
 =head1 OPEN ISSUES
 
-Problems were found with the use of the Inline C module. The problem is caused
-by user and root rights. Until this issue is resolved, the POSIX module is used
-instead of the C code.   
+Procedures must be implemented so that System Console and logins via SSH can be
+uniquely recognised. Both currently report System Console as the result.
+
+Subroutines must be implemented to check whether the necessary system programmes
+and the necessary system files are available. 
 
 =head1 KNOWN BUGS
 
-Not known yet
+The terminal emulator WezTerm could not be identified. Reason is a missing pseudo
+terminal slave. This has to be checked, why this is the case.
+
+=head1 ERROR CODES
+
+The package returns no error codes at the moment.
+
+=head1 NOTES
+
+Problems were found with the use of the Inline C module. The problem is caused
+by user and root rights. Until this issue is resolved, the POSIX module is used
+instead of the C code.
 
 =head1 ABSTRACT
 
@@ -729,18 +795,18 @@ runs from within the system console, the output returns the system console.
 
 =head1 SEE ALSO
 
-users(1) - Linux manual page
-
-shells(5) - Linux manual page
-
-passwd(5) - Linux manual page
-
 ps(1) - Linux manual page
 
+users(1) - Linux manual page
+
 which(1) - Linux manual page
 
 grep(1) - Linux manual page
 
+shells(5) - Linux manual page
+
+passwd(5) - Linux manual page
+
 =head1 AUTHOR
 
 Dr. Peter Netz, E<lt>ztenretep@cpan.orgE<gt>
